/*#######################################################
 *
 *   Maintained by Gregor Santner, 2017-
 *   https://gsantner.net/
 *
 *   License of this file: Apache 2.0 (Commercial upon request)
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
#########################################################*/
package net.gsantner.markor.util;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.print.PrintJob;
import android.support.annotation.RequiresApi;
import android.text.TextUtils;
import android.webkit.WebView;

import net.gsantner.markor.R;
import net.gsantner.markor.activity.MainActivity;
import net.gsantner.markor.activity.openeditor.OpenEditorFromShortcutOrWidgetActivity;
import net.gsantner.markor.model.Document;

public class ShareUtil extends net.gsantner.opoc.util.ShareUtil {
    public ShareUtil(Context context) {
        super(context);
        setChooserTitle(_context.getString(R.string.share_to_arrow));
    }

    public void createLauncherDesktopShortcut(final Document document) {
        // This is only allowed to call when direct file access is possible!!
        // So basically only for java.io.File Objects. Virtual files, or content://
        // in private/restricted space won't work - because of missing permission grant when re-launching
        if (document != null && !TextUtils.isEmpty(document.getTitle())) {
            final boolean isDir = document.getFile().isDirectory();
            final Class<?> klass = isDir ? MainActivity.class : OpenEditorFromShortcutOrWidgetActivity.class;
            final Intent intent = new Intent(_context, klass).setData(Uri.fromFile(document.getFile()));
            final int iconRes = isDir ? R.mipmap.ic_shortcut_folder : R.mipmap.ic_shortcut_file;
            super.createLauncherDesktopShortcut(intent, iconRes, document.getTitle());
            // Toast.makeText(_context, R.string.tried_to_create_shortcut_for_this_notice, Toast.LENGTH_LONG).show();
        }
    }

    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    @SuppressWarnings("deprecation")
    public PrintJob printOrCreatePdfFromWebview(WebView webview, Document document, boolean... landscape) {
        String jobName = String.format("%s (%s)", document.getTitle(), _context.getString(R.string.app_name_real));
        return super.print(webview, jobName, landscape);
    }

    public void showMountSdDialog(Activity... activity) {
        showMountSdDialog(R.string.mount_storage, R.string.application_needs_access_to_storage_mount_it, R.drawable.mount_sdcard_help, activity);
    }
}
